<?php
/******************************************************************************/
/* AEL Sarl : Web hosting, datacenter, gestion et comptabilité déportées      */
/* Copyright (C) 2008 Affaires En Ligne S.A.R.L.                              */
/* Contact : contact@affaires-en-ligne.fr                                     */
/* Web : http://www.spidboutic.fr                                             */
/*                                                                            */
/* This program is free software; you can redistribute it and/or              */
/* modify it under the terms of the GNU General Public License                */
/* as published by the Free Software Foundation; either version 2             */
/* of the License, or (at your option) any later version.                     */
/*                                                                            */
/* This program is distributed in the hope that it will be useful,            */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of             */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the               */
/* GNU General Public License for more details.                               */
/*                                                                            */
/* You should have received a copy of the GNU General Public License          */
/* along with this program; if not, write to the Free Software                */
/* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */
/******************************************************************************/

/*
 * Created on 4 oct. 07 by AEL Sarl for
 * 
 * osCommerce, Open Source E-Commerce Solutions
 * http://www.oscommerce.com
 *
 * Copyright (c) 2003 osCommerce
 *
 * Released under the GNU General Public License
 * 
 */

require_once("OpensiStructure.php");

class OrderTotal {
	
	/**
	 * Build an OrderTotal instance
	 * 
 	 * @param	array	$orders_total_values	array of associ
	 */
	public function __construct() {
		$this->ot_shipping = 0;
		$this->ot_tax = 0;
		$this->ot_total = 0;
		$this->excluded_tax = 0;
	}
	
	/**
	 * Calculate excluded tax value
	 */
	private function calculateExcludedTax() {
		$this->excluded_tax = $this->ot_total - $this->ot_tax;
	}
	
	/**
	 * Set an instance attribute
	 * 
	 * @param	array	$order_total_row	associated array of an orders_total record
	 */
	public function setAttribute($order_total_row) {
		$ot_class = $order_total_row['class'];
		if ($ot_class == 'ot_shipping' || $ot_class == 'ot_tax' || $ot_class == 'ot_total') {
			$this->$ot_class = $order_total_row['value'];
			if ($ot_class == 'ot_tax' || $ot_class == 'ot_total') {
				$this->calculateExcludedTax();
			}
		}
	}
	
}

class OpensiOrder implements OpensiStructure {
	
	const ID = 0;
	const MODE_PAIEMENT = 1;
	const FRAIS_PORT = 2;
	const TOTAL_TTC = 3;
	const TOTAL_HT = 4;
	const LISTE_ARTICLES = 5;
	const CLIENT_LIV = 6;
	const CLIENT = 7;
	const STATUT = 8;
	const REMISE = 9;
	const TAUX_REMISE = 10;
	const DATE = 11;
	const STATUT_PAIE = 12;
	const EMAIL_MANAGER = 13;
	const MODE_EXPEDITION = 14;
	const COMMENTAIRES = 15;
	const COMMENTAIRES_2 = 16;
	const VALIDATION = 17;
	const EDITION_COM = 18;
	const EDITION_TTC = 19;
	const TYPE_VENTE = 20;
	const CODE_PORTE = 21;
	const NUM_TRANSACTION = 22;
	const ORIGINE = 23;
	const INFOS_COMMANDE = 24;
	const NUM_COLIS = 25;
	const LIEN_COLIS = 26;
	const CODE_PAYS_LIV = 27;
	
	private $order_payment_method;
	private $order_payment_status;
	private $order_sale_type;
	private $order_date_purchased;
	private $orders_status;
	private $orders_validation;
	
	/**
	 * Build an order
	 * @param	array					$order				an order, with some order parameters
	 * @param 	OpensiStructureVector	$products_vector	vector of OpensiOrderProduct
	 * @param	OpensiPerson			$delivery_customer	the delivery person
	 * @param	OpensiCustomern			$customer			the customer
	 */
	public function __construct($order, $products_vector=null, $delivery_customer=null, $customer=null) {
		
		$this->orders_id = $order[self::ID];
		$this->setOrdersStatus($order[self::STATUT]);
		$this->setOrdersValidation($order[self::VALIDATION]);
		$this->setPaymentMethod($order[self::MODE_PAIEMENT]);
		$this->setPaymentStatus($order[self::STATUT_PAIE]);
		$this->setSaleType($order[self::TYPE_VENTE]);
		$this->setDatePurchased($order[self::DATE]);
		
		$this->order = $order;
		$this->order_customer = $customer;
		$this->order_delivery_customer = $delivery_customer;
		$this->order_products = $products_vector;		
	}
	
	/**
	 * Set the orders_payment_method attribute
	 * 
	 * @param	string	$method		the method to set as a valid order payment method
	 */
	public function setPaymentMethod($method) {
		$modes = array(OSC_CHEQUE_PAYMENT_METHOD => 'Cheque',
		               OSC_CARD_PAYMENT_METHOD => 'Carte bancaire',
		               OSC_TRANSFER_PAYMENT_METHOD => 'Virement',
		               OSC_DELIVERY_PAYMENT_METHOD => 'Cheque');
		foreach ($modes as $osc_mode => $opensi_mode) {		               
			if (strpos($method, $osc_mode) !== false) {
				$this->order_payment_method = $opensi_mode;
				break;
			}
		}
		$this->order_payment_method = "";
	}
	
	/**
	 * Set the payment status attribute
	 * 
	 * @param	string	$status	the status to set
	 */
	public function setPaymentStatus($status) {
		$this->order_payment_status = ($status > 1 ? 1 : 0);
	}
	
	/**
	 * Set the sale type attribute
	 * 
	 * @param	string	$country	country name used to set the sale type
	 */
	public function setSaleType($country) {
		$eu = array('Germany', 'Austria', 'Belgium', 'Bulgaria', 'Cyprus', 'Denmark', 'Spain',
		            'Estonia', 'Finland', 'France', 'Greece', 'Hungary', 'Ireland', 'Italy',
		            'Latvia'. 'Lithuania', 'Luxembourg', 'Malta', 'Netherlands', 'Poland', 'Portugal',
                    'Romania', 'United Kingdom', 'Slovakia', 'Slovenia', 'Sweden', 'Czech Republic');
		if ($country == 'France')
			$this->order_sale_type = "N";
		else if (in_array($country, $eu))
			$this->order_sale_type = "U";
		else
			$this->order_sale_type = "I";			
	}
	
	/**
	 * Set the order status attribute
	 * 
	 * @param int/string	$status	order status value to set
	 */
	public function setOrdersStatus($status) {
		if (preg_match('/\d/', $status)) // assume it is an OsCommerce status value
			$this->orders_status = $status;
		else {
			$status_query = tep_db_query("SELECT Orders_Status_Id FROM OPENSI_ORDERS_STATUS" .
			                             " WHERE Opensi_Orders_Status_Name = '" . $status . "'");
			$status_array = tep_db_fetch_array($status_query);
			if (tep_not_null($status_array))
				$this->orders_status = $status_array['Orders_Status_Id'];
			else
				$this->orders_status = 0;			                             
		}
	}
	
	/**
	 * Set the orders_validation attribute
	 * 
	 * @param	int	$validation		the value to set
	 */
	public function setOrdersValidation($validation) {
		$validation_query = tep_db_query("SELECT Opensi_Orders_Status_Id FROM OPENSI_ORDERS_STATUS" .
			                             " WHERE Orders_Status_Id = " . $validation);
		$validation_array = tep_db_fetch_array($validation_query);
		if (tep_not_null($validation_array))
			$this->orders_validation = $validation_array['Opensi_Orders_Status_Id'];
		else
			$this->orders_validation = 0;
	}
	
	/**
	 * Set the date purchased attribute
	 * 
	 * @param	string	$date_purchased		purchased date to set as date_purchased attribute
	 */
	public function setDatePurchased($date_purchased) {
	 	$this->order_date_purchased = opensi_datetime($date_purchased);
	 }
	
	/**
	 * Get the orders_payment_method attribute
	 * 
	 * @return	string	payment method value
	 */
	public function getPaymentMethod() {
		return $this->order_payment_method;
	}
	
	/**
	 * Get the payment_status attribute
	 * 
	 * @return	int	payment status value
	 */
	public function getPaymentStatus() {
		return $this->order_payment_status;
	}
	
	/**
	 * Get the sale type attribute
	 * 
	 * @return	string	sale type value
	 */
	public function getSaleType() {
		return $this->order_sale_type;
	}
	
	/**
	 * Get the date_purchased attribute
	 * 
	 * @return	string	date_purchased attribute value
	 */
	public function getDatePurchased() {
		return $this->order_date_purchased;
	}
	
	/**
	 * Get the orders status attribute
	 * 
	 * @return int	the order status attribute
	 */
	public function getOrdersStatus() {
		return $this->orders_status;
	}
	
	/**
	 * Get the orders validation attribute
	 * 
	 * @return	int	orders validation attribute
	 */
	public function getOrdersValidation() {
		return $this->orders_validation;
	}
	
	/**
	 * Update the orders_status field
	 * 
	 * @return	an array for the update operation
	 */
	public function updateOrderStatus() {
		$update_order_query = tep_db_query(
		                         "UPDATE " . TABLE_ORDERS .
		                         " SET orders_status = " . $this->orders_status .
		                         " WHERE orders_id = " . (0 + $this->orders_id));
		// store the query status
		$op_status = tep_db_affected_rows();		                         
		// update history if orders_status is modified
		if ($op_status > 0) {
			$update_history_query = tep_db_query(
			                           "INSERT INTO " . TABLE_ORDERS_STATUS_HISTORY .
			                           " (orders_id, orders_status_id, date_added, customer_notified) VALUES (" .
			                           (0 + $this->orders_id) . ", " .
			                           $this->orders_status . ", NOW(), 0)");
		}		                         
		return (tep_db_affected_rows() >= 0 ? 
		        	array(UPDATE_OK, $this->orders_id) :
		        	array(UPDATE_KO, $this->orders_id));		                         
	}
	
	/**
	 * Prepare the upload of the OpensiOrder object
	 * 
	 * @return array	prepared for upload object
	 */
	public function prepareUpload() {
		$prepared = $this->order;
		
		$prepared[self::MODE_PAIEMENT] = $this->order_payment_method;
		$prepared[self::STATUT_PAIE] = $this->order_payment_status;
		$prepared[self::TYPE_VENTE] = $this->order_sale_type;
		$prepared[self::DATE] = $this->order_date_purchased;
		$prepared[self::STATUT] = $this->orders_status;
		$prepared[self::VALIDATION] = $this->orders_validation;
		
		$prepared_customer = ($this->order_customer ? $this->order_customer->prepareUpload() : array());
		$prepared[self::CLIENT] = $prepared_customer;
		
		$prepared_delivery_customer = ($this->order_delivery_customer ? $this->order_delivery_customer->prepareUpload() : array());
		$prepared[self::CLIENT_LIV] = $prepared_delivery_customer;
		
		$prepared_order_products = ($this->order_products ? $this->order_products->prepareUpload() : array());
		$prepared[self::LISTE_ARTICLES] = $prepared_order_products;
		
		return $prepared;
	}
}
?>
