<?php
/******************************************************************************/
/* AEL Sarl : Web hosting, datacenter, gestion et comptabilité déportées      */
/* Copyright (C) 2008 Affaires En Ligne S.A.R.L.                              */
/* Contact : contact@affaires-en-ligne.fr                                     */
/* Web : http://www.spidboutic.fr                                             */
/*                                                                            */
/* This program is free software; you can redistribute it and/or              */
/* modify it under the terms of the GNU General Public License                */
/* as published by the Free Software Foundation; either version 2             */
/* of the License, or (at your option) any later version.                     */
/*                                                                            */
/* This program is distributed in the hope that it will be useful,            */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of             */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the               */
/* GNU General Public License for more details.                               */
/*                                                                            */
/* You should have received a copy of the GNU General Public License          */
/* along with this program; if not, write to the Free Software                */
/* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */
/******************************************************************************/

/*
 * Created on 27 sept. 07 by AEL Sarl for
 * 
 * osCommerce, Open Source E-Commerce Solutions
 * http://www.oscommerce.com
 *
 * Copyright (c) 2003 osCommerce
 *
 * Released under the GNU General Public License
 * 
 */
 

include("includes/scheduled.php");
require_once("includes/classes/OpensiProduct.php");
require_once("includes/classes/ImageDownload.php");
require_once("includes/classes/OpensiCustomer.php");
require_once("includes/classes/OpensiOrder.php");
require_once("includes/classes/OpensiOrderProduct.php");
require_once("includes/classes/OpensiStructureVector.php");

/**
 * Adds new products and updates known products in the database
 * 
 * @param 	array	$products_list an array of products values arrays
 * @return	array	an OpenSI report
 */
function ajouterArticles($products_list) {
	
	$final_report = array();
		
	foreach ($products_list as $product) {
		$opensi_product = new OpensiProduct($product);
		
		// We must first know if the product is known, or not
		// before choosing the rightest operation, update or insert
		if($opensi_product->isKnown === null) {
			$final_report[] = array(AUTRE_KO, ERR_DATABASE);
			continue;
		}
		if ($opensi_product->isKnown)
			$opensi_product->state = new OpensiProductUpdateState();
		else
			$opensi_product->state = new OpensiProductInsertState();
		
		$report = $opensi_product->updateOscProduct();
		if ($report)
			$final_report[] = $report;
		else
			$final_report[] = array(INSERT_OK,
			                        $opensi_product->name . ' [ ' . $opensi_product->opensi_web_id . ' ]');
	}
	tep_db_query("INSERT INTO WEB_MANAGER_OPERATIONS VALUES ('ajoutArticles', NOW(), 0)" .
                 " ON DUPLICATE KEY UPDATE Web_Manager_Last_Modified = NOW()");
	
	return array($final_report);
}

/**
 * Sends an image to the OsCommerce web site
 * 
 * @param string	$opensi_web_id		opensi web identifier of the article
 * @param int		$size_small			size of the small image
 * @param string	$content_small		content of the small image, base64 encoded
 * @param string 	$name_small			name of the small image
 * @param int		$size_big			size of the big image
 * @param string	$content_big		content of the big image, base64 encoded
 * @param string	$name_big			name of the big image
 */
function ajouterImageAuSite($opensi_web_id, $size_small, $content_small, $name_small,
					  $size_big, $content_big, $name_big) {
	$download = new ProductImageDownload($opensi_web_id, $name_big, $size_big, $content_big);
	if ($download->download())
		$download->updateImageName();
}


/**
 * Update products infos as manufacturer, weight, category...except price and stock quantity
 * This function updates known products, but can adds new product
 * Price and Quantity will be updated later, by other functions
 * This function sets the image name to '' (empty string)
 * 
 * @param	array	$infos	an array of products info arrays
 * @return 	array	an OpenSI report
 */
function majInfos($infos) {
	
	$final_report = array();
		
	foreach ($infos as $info_product) {
		$opensi_product = new OpensiProduct($info_product);
		
		// We must first know if the product is known, or not
		// before choosing the rightest operation, update or insert
		if($opensi_product->isKnown === null) {
			$final_report[] = array(AUTRE_KO, ERR_DATABASE);
			continue;
		}
		if ($opensi_product->isKnown)
			$opensi_product->state = new OpensiProductInfosUpdateState();
		else
			$opensi_product->state = new OpensiProductInfosInsertState();
		
		$report = $opensi_product->updateOscProduct();
		
		if ($report)
			$final_report[] = $report;
		else
			$final_report[] = array(INSERT_OK,
			                        $opensi_product->name . ' [ ' . $opensi_product->opensi_web_id . ' ]');
	}
	tep_db_query("INSERT INTO WEB_MANAGER_OPERATIONS VALUES ('majInfos', NOW(), 0)".
                 " ON DUPLICATE KEY UPDATE Web_Manager_Last_Modified = NOW()");
	
	return array($final_report);
}

/**
 * Update quantity value for an identified product
 * 
 * @param 	array		$products	array of products values quantity and web_id
 * @return	array		an OpenSI report
 */
function majStocks($products) {
	
	$final_report = array();
	/*
	 * $products is an array of arrays
	 * Each array in $product is an array of quantity at index 0
	 * and opensi_web_id at index 1
	 */
	foreach ($products as $product) {
		$update_stock_query = tep_db_query(
		                         "UPDATE " . TABLE_PRODUCTS .
		                         " SET products_quantity = " . $product[0] .
		                         " WHERE products_id = " .
                                 "(SELECT products_id FROM PRODUCTS_TO_OPENSI" .
                                 		" WHERE Opensi_Web_id = '" . $product[1] . "')");
		if (tep_db_affected_rows() >= 0)
			$report = array(UPDATE_OK, $product[1]);
		else
			$report = array(UPDATE_KO, $product[1]);			                                 		
		$final_report[] = $report;
	}
	tep_db_query("INSERT INTO WEB_MANAGER_OPERATIONS VALUES ('majStocks', NOW(), 0)".
                 " ON DUPLICATE KEY UPDATE Web_Manager_Last_Modified = NOW()");
             
	return array($final_report);
}

/**
 * Update quantity value for an identified product
 * 
 * @param 	array		$products	array of products
 * @return	array		an OpenSI report
 */
function majPrix($products) {
	
	$final_report = array();
	
	foreach ($products as $product) {
		$opensi_product = new OpensiProduct($product);		
		$final_report[] = $opensi_product->updateProductPrice();
	}
	tep_db_query("INSERT INTO WEB_MANAGER_OPERATIONS VALUES ('majPrix', NOW(), 0)".
                 " ON DUPLICATE KEY UPDATE Web_Manager_Last_Modified = NOW()");
	
	return array($final_report);
}

/**
 * Update the orders status an identified order
 * 
 * @param 	array		$orders	array of orders
 * @return	array		an OpenSI report
 */
function majStatutCom($orders) {
	
	$final_report = array();
	
	foreach ($orders as $order) {
		$opensi_order = new OpensiOrder($order);		
		$final_report[] = $opensi_order->updateOrderStatus();
	}
	tep_db_query("INSERT INTO WEB_MANAGER_OPERATIONS VALUES ('majStatutCom', NOW(), 0)".
                 " ON DUPLICATE KEY UPDATE Web_Manager_Last_Modified = NOW()");
	
	return array($final_report);
}

/**
 * Upload all customers
 * 
 * @return	array	an OpenSI report with a flag indicating weather the function should be
 *                  called once again or not.
 */
function uploadClients() {
	
	$site_customers = array();
	$final_report = array();
	
	$max_customers_id = get_web_manager_op_counter('uploadClients');
	
	$customers_query = tep_db_query(
	                    "SELECT cu.customers_id, cu.customers_email_address, cu.customers_telephone, cu.customers_fax," .
	                    " cu.customers_gender, cu.customers_lastname, cu.customers_firstname, ab.entry_company," .
	                    " ab.entry_street_address, ab.entry_suburb, '' AS address_3, ab.entry_postcode, ab.entry_city, co.countries_name" .
	                    " FROM " . TABLE_CUSTOMERS . " cu, " .
	             		TABLE_COUNTRIES . " co, " . TABLE_ADDRESS_BOOK . " ab" .
	             		" WHERE ab.entry_country_id = co.countries_id" .
	             		" AND cu.customers_default_address_id = ab.address_book_id" .
	             		" AND cu.customers_id = ab.customers_id" .
	             		" AND cu.customers_id > " . $max_customers_id .
                        " ORDER BY cu.customers_id ASC" .
                        " LIMIT 0," . (OPENSI_FETCH_LIMIT + 1));
    
    $has_more = (tep_db_num_rows($customers_query) > OPENSI_FETCH_LIMIT);
    
	while ($customer = tep_db_fetch_row($customers_query)) {
		if (count($site_customers) == OPENSI_FETCH_LIMIT)
    		break;

		$opensi_customer = new OpensiCustomer(array_slice($customer, 0, 4));
		$opensi_person =  new OpensiPerson(array_slice($customer, 4));
		$opensi_customer->customers_person = $opensi_person;
		$prepared = $opensi_customer->prepareUpload();
		
		if (is_array($prepared) &&
		    count($prepared) == 5 && 
		    $prepared[0] == $customer[0] && 
		    is_array($prepared[4]) &&
		    count($prepared[4]) == 10)
			$final_report[] = array(SELECT_OK, $opensi_person->firstname . " " . $opensi_person->lastname);
		else {
			$final_report[] = array(SELECT_KO, $opensi_person->firstname . " " . $opensi_person->lastname);
			$prepared = array();
		}
		
		$site_customers[] = $prepared;
	}
	
	// reset the counter when finished, for the next call
	if (!$has_more)
    	tep_db_query("INSERT INTO WEB_MANAGER_OPERATIONS VALUES ('uploadClients', NOW(), 0)".
                     " ON DUPLICATE KEY UPDATE Web_Manager_Last_Modified = NOW(), Web_Manager_Op_Counter = 0");
	else
		tep_db_query("INSERT INTO WEB_MANAGER_OPERATIONS VALUES".
                     " ('uploadClients', NOW(), " . $opensi_customer->customers_id . ")" .
                     " ON DUPLICATE KEY UPDATE" .
                     " Web_Manager_Last_Modified = NOW()," .
                     " Web_Manager_Op_Counter = VALUES(Web_Manager_Op_Counter)");
	
	tep_db_close();
    return array($final_report, array($site_customers, $has_more));
}

/**
 * Upload orders
 * @return	array	an OpenSI report with a flag indicating weather the function should be
 *                  called once again or not.
 */
function uploadCommandes() {
	$site_orders = array();
	$final_report = array();
	
	$max_orders_id = get_web_manager_op_counter('uploadCommandes');
		                              
	$orders_query = tep_db_query(
							"SELECT o.orders_id, o.payment_method, o.orders_status, o.date_purchased," .
	    					" cu.customers_id, cu.customers_email_address, cu.customers_telephone, cu.customers_fax," .
	             		    " cu.customers_gender, cu.customers_lastname," .
	             		    " cu.customers_firstname, ab.entry_company, ab.entry_street_address, ab.entry_suburb," .
                 		    " '' AS address_3, ab.entry_postcode, ab.entry_city, co.countries_name, co.countries_iso_code_3," .
                 		    " o.delivery_name, o.delivery_company, o.delivery_street_address, o.delivery_suburb," .
                 		    " '' as delivery_address_3, o.delivery_postcode, o.delivery_city, o.delivery_country" .
	             		    " FROM " . TABLE_ORDERS . " o, " . TABLE_CUSTOMERS . " cu, " .
	             		    TABLE_COUNTRIES . " co, " . TABLE_ADDRESS_BOOK . " ab" .
	             		    " WHERE ab.entry_country_id = co.countries_id" .
	             		    " AND cu.customers_default_address_id = ab.address_book_id" .
	             		    " AND cu.customers_id = ab.customers_id" .
	             		    " AND cu.customers_id = o.customers_id" .
                        	" AND o.orders_id > " . $max_orders_id .
                        	" ORDER BY o.orders_id ASC" .
                        	" LIMIT 0," . (OPENSI_FETCH_LIMIT + 1));
    
    $has_more = (tep_db_num_rows($orders_query) > OPENSI_FETCH_LIMIT);
                  	
	while ($order = tep_db_fetch_row($orders_query)) {
		if (count($site_orders) == OPENSI_FETCH_LIMIT)
    		break;
    	
		// fields from customers_id at index 4
		// to customers_fax at index 7
		$opensi_customer = new OpensiCustomer(array_slice($order, 4, 4));
		// from customers_civility at index 8 to co.countries_name at index 17
		$opensi_customer_person =  new OpensiPerson(array_slice($order, 8, 10));
		$opensi_customer->customers_person = $opensi_customer_person;
		
		// check now the delivery
		// delivery_name is at index 19
		if (tep_not_null($order[19])) { 
			$delivery = array_slice($order, 19, 8);
			// prepend empty values for civility and lastname
			// firstname is the full name and will be modified later
			array_unshift($delivery, '', '');
			$opensi_delivery_person = new OpensiPerson($delivery);
			$opensi_delivery_person->setFullCivility($opensi_delivery_person->firstname);
			// build a delivery customer with id 0, it does not matter
			// and empty email, tel, fax cause we don't have in it basic OsCommerce
			$opensi_delivery_customer = new OpensiCustomer(array(0, '', '', ''), $opensi_delivery_person);
		}	
		else // assume it's the same person as the customer
			$opensi_delivery_customer = &$opensi_customer;
		
		// fetch amounts values
		$amounts_query = tep_db_query(
		                        "SELECT class, SUM(value) as value FROM " . TABLE_ORDERS_TOTAL .
		                        " WHERE orders_id = " . $order[0] .
		                        " GROUP BY class");
		$ot = new OrderTotal;
		while ($amount = tep_db_fetch_array($amounts_query))
			$ot->setAttribute($amount);
		
		/* build the 27 items order_array with
		 * - orders_id, payment_method, shipping, included_tax, excluded_tax
		 * - next 3 indexes are reserved for products list, delivery and customer
		 * - the next field, which is called STATUT, is not used on upload
		 *   (hardcoded by OpenSI UploadCommandes.java and depending of orders_status)
		 * - the two next fields are for discount, and discount rate : no discount in OsCommerce !
		 * - next field is a date field : it is used for payment date (if status permits it),
		 *   validation date, and orders_date. Need a format dd/mm/yyyy/hh/mm
		 * - next field is for payment status: assume that order is paid when processing it (orders_status > 1)
		 * - next field is manager email: we use the store owner email
		 * - next field is for transport mode : we cannot get the information, so ""
		 * - two fields now for comments, found in orders_status_history table
		 * - next field is for order validation : we use orders_status
		 * - two next fields indicating the document edition, set to 0
		 * - next field for orders type: we use the country name of the delivery site, but
		 *   we get it from the delivery_person object
		 * - next field is CODE_PORTE, don't know what it is
		 * - next field is related to the transaction number (credit card)
		 * - next field is ORIGINE, dunno what it is
		 * - then field INFOS_COMMANDE, dunno...
		 * - then two fields for shipping tracking : shipping number, and shipping tracking web link.
		 *   Needs a shipping tracking contrib
		 * - last field for delivery country code : we use the countries_iso_code_3 of the countries table
		 *   Those last 3 fields are not COMMANDE_WEB fields in the OpenSI database.
		 */
		$comments_query = tep_db_query(
		                      "SELECT comments FROM " . TABLE_ORDERS_STATUS_HISTORY .
		                      " WHERE orders_id = " . $order[0] .
		                      " ORDER BY orders_status_history_id ASC LIMIT 2");
		$comments = array("", "");
		$i = 0;
		while ($comment = tep_db_fetch_array($comments_query)) {
			$comments[$i] = $comment['comments'];
			$i++;
		}           
		$order_array = array($order[0], $order[1], $ot->ot_shipping, $ot->ot_total, $ot->excluded_tax,
		                     null, null, null, '', 0, 0, $order[3], $order[2], STORE_OWNER_EMAIL_ADDRESS,
		                     "", $comments[0], $comments[1], $order[2],
		                     0, 0, $opensi_delivery_customer->customers_person->country,
		                     '', '', '', '', '', '', $order[18]);
		
		/*
		 * Prepare now the orders_products list
		 */
		$order_products_vector = new OpensiStructureVector();
		$order_product_query = tep_db_query(
		                           "SELECT po.Opensi_Web_Id, op.final_price, op.products_quantity, 0 AS discount," .
		                           " op.products_name, op.products_tax, '' AS comment" .
		                           " FROM PRODUCTS_TO_OPENSI po, " . TABLE_ORDERS_PRODUCTS . " op" .
		                           " WHERE op.products_id = po.Products_Id" .
		                           " AND op.orders_id = " . $order[0]);
		while ($order_product = tep_db_fetch_row($order_product_query))
			$order_products_vector->addStructure(new OpensiOrderProduct($order_product));
				                           
		$opensi_order = new OpensiOrder($order_array,
		                                $order_products_vector,
		                                $opensi_delivery_customer,
		                                $opensi_customer);
		$prepared = $opensi_order->prepareUpload();
		
		/*
		 * $prepared[5] is orders_products list
		 * $prepared[6] is the delivery
		 * $prepared[7] is the customer
		 */
		if (is_array($prepared) && count($prepared) == 28 && 
		    $prepared[0] == $order[0] && 
		    is_array($prepared[5]) && count($prepared[5] > 0) &&
		    is_array($prepared[6]) && count($prepared[6]) == 5 &&
		    is_array($prepared[7]) && count($prepared[7]) == 5)
			$final_report[] = array(SELECT_OK, $opensi_order->orders_id);
		else {
			$final_report[] = array(SELECT_KO, $opensi_order->orders_id);
			$prepared = array();
		}
		
		$site_orders[] = $prepared;
	}
	
	// set the counter to the max value when finished, for the next call
	// but only if an operation has been done
	if (isset($opensi_order))
		tep_db_query("INSERT INTO WEB_MANAGER_OPERATIONS VALUES".
                 	 " ('uploadCommandes', NOW(), " . $opensi_order->orders_id . ")" .
                 	 " ON DUPLICATE KEY UPDATE" .
                 	 " Web_Manager_Last_Modified = NOW()," .
                 	 " Web_Manager_Op_Counter = VALUES(Web_Manager_Op_Counter)");  	
    
	tep_db_close();
    return array($final_report, array($site_orders, $has_more));
}

/**
 * Dummy function for update of invoices
 */
function majInfosFacture($invoices) {
	
	$final_report = array();

	foreach ($invoices as $invoice) {
		$final_report[] = array(UPDATE_OK, $invoice[0]); // $invoice[0] is the orders_id
	}
	
	return array($final_report);
}
?>
